%Author: Serge Shikher, Suffolk University.
%This program accompanies Shikher, S. "Capital, technology, and specialization in the neoclassical model".

function res=cf(type,name,descrip,dsname,nchange,ichange,jchange,reqchangefactor,base)

%Getting variables from the data structure
N=base.N; %number of countries
J=base.J; %number of industries
alpha=base.alpha; %capital share
beta=base.beta; %labor share
etaM=base.etaM; %industry share in intermediate good (input-output structure)
phi=base.phi; %consumption share in GDP
phiM=base.phiM; %consumption share in manufacturing GDP
theta=base.theta; %technology parameter
varphi=base.varphi; %share of nonmanufacturing in GDP
VAnO=varphi.*base.VAn; %initial nonmanufacturing income
T=base.T; %technology parameter
d=base.d; %trade costs
Ln=base.Ln; %country labor force
Kn=base.Kn; %country capital stock

%Setting up initial values
c0=base.c; %cost of production
L0=base.L; %industry labor employment
w0=base.w; %country wage
K0=base.K; %industry capital stocks
r0=base.r; %country rate of return

fprintf('-----------------------------------------------------------------------\n');
fprintf('>>> Experiment: %s.\n',descrip);
fprintf('>>> Starting experiment. Type: %s. Name: %s. Dataset: %s.\n',type,name,dsname);
markstart=now;

MAXSTEPS=3; %change this number to control the convergence of nonmanufacturing income
STEP=0;
stopflag=0;
while STEP<MAXSTEPS & stopflag==0
    STEP=STEP+1;
    if STEP==2
        Knbank=base.Kn;
        Lnbank=base.Ln;
        Tbank=base.T;
        changefactor=reqchangefactor;
        if strcmp(type,'Kn')
            Kn(nchange,STEP)=Knbank(nchange).*changefactor;
        elseif strcmp(type,'T')
            T(nchange,jchange)=Tbank(nchange,jchange).*changefactor;
        end;
    end;
    markstepstart=now;
    fprintf('-----------------------------------------------------------------------\n');
    fprintf('>>> Starting step number %d.\n',STEP);

    %Running simulation
    [c(:,:,STEP),p(:,:,STEP),rho(:,:,STEP),pi(:,:,:,STEP),L(:,:,STEP),w(:,STEP),K(:,:,STEP),r(:,STEP),exitflag(STEP),output(STEP)]=...
        simulatesystem(alpha,beta,etaM,phi,theta,T,d,Ln,Kn(:,STEP),VAnO(:,STEP),c0,L0,w0,K0,r0,N,J);

    %Calculating derived variables
    Q(:,:,STEP)=(repmat(w(:,STEP),[1,J]).*L(:,:,STEP))./repmat(beta',[N,1]); %industry output
    VA(:,:,STEP)=repmat(w(:,STEP),[1,J]).*L(:,:,STEP)+repmat(r(:,STEP),[1,J]).*K(:,:,STEP); %industry income
    pn(:,STEP)=prod(p(:,:,STEP).^phiM,2); %country (manufacturing) consumption price level
    VAnM(:,STEP)=sum(VA(:,:,STEP),2); %manufacturing income
    VAn(:,STEP)=VAnM(:,STEP)+VAnO(:,STEP); %country income
    VAnO(:,STEP+1)=varphi.*VAn(:,STEP);
    Secon(:,:,STEP)=VA(:,:,STEP)./repmat(VAn(:,STEP),[1,J]); %industry share in economy
    S(:,:,STEP)=VA(:,:,STEP)./repmat(VAnM(:,STEP),[1,J]); %industry share in manufacturing
    logA(:,:,STEP)=log(VA(:,:,STEP))-repmat((alpha./(alpha+beta))',[N,1]).*log(K(:,:,STEP))-repmat((beta./(alpha+beta))',[N,1]).*log(L(:,:,STEP)); %TFP calculated using nominal income
    Kn(:,STEP)=sum(K(:,:,STEP),2);
    Kn(:,STEP+1)=Kn(:,STEP); %total capital stock is fixed
    CK=repmat((ones(J,1)-alpha-beta)./alpha,[1,J]).*etaM;
    Z(:,:,STEP)=squeeze(sum(repmat(shiftdim(CK,-1),[N,1,1]).*repmat(repmat(r(:,STEP),[1,J]).*K(:,:,STEP),[1,1,J]),2)); %spending on intermediate goods made by industry
    C(:,:,STEP)=repmat(VAn(:,STEP),[1,J]).*phi; %spending on consumption goods made by industry
    Y(:,:,STEP)=C(:,:,STEP); %spending on final goods made by industry
    X(:,:,STEP)=Z(:,:,STEP)+Y(:,:,STEP); %spending on all (intermediate and final) goods made by industry
    IM(:,:,STEP)=squeeze(sum(pi(:,:,:,STEP).*repmat((ones(19)-eye(19)),[1,1,8]),2)).*X(:,:,STEP); %spending on imports made by industry
    IMM(:,STEP)=sum(IM(:,:,STEP),2); %total manufacturing imports
    EX(:,:,STEP)=Q(:,:,STEP)-X(:,:,STEP)+IM(:,:,STEP); %spending on exports made by industry
    EXM(:,STEP)=sum(EX(:,:,STEP),2); %total manufacturing exports
    IMMw(STEP)=sum(IMM(:,STEP)); %total world manufacturing imports (world trade)
    EXMw(STEP)=sum(EXM(:,STEP)); %total world manufacturing exports (world trade), should be the same as IMMw
    
    markstepend=now;
    steptime=markstepend-markstepstart;
    
    if exitflag(STEP)<=0
        stopflag=1;
        fprintf('>>> This step did not complete successfully (%d/%d).\n',errorcount,maxallowederrors);
    else
        fprintf('>>> The step has completed successfully. It took %s.\n',datestr(markstepend-markstepstart,13));
        c0=c(:,:,STEP);
        L0=L(:,:,STEP);
        w0=w(:,STEP);
        K0=K(:,:,STEP);
        r0=r(:,STEP);
    end;
end;

markend=now;
fprintf('>>> The simulation finished running. It simulated %d step(s). It took %s or %s per step.\n',...
    STEP,datestr(markend-markstart,13),datestr((markend-markstart)./STEP,13));

storeresults;
